#!/usr/bin/env python

import sys
from unittest import TestCase

def peek_token(p):
    tok = p.get_token()
    p.unget_token(tok)
    return tok

# XXX .tags()

class PullParserTests(TestCase):
    def data_and_file(self):
        from StringIO import StringIO
        data = """<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN"
"http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<title an=attr>Title</title>
</head>
<body>
<p>This is a data <img alt="blah"> &amp; that was an entityref and this &#097; is
a charref.  <blah foo="bing" blam="wallop">.
<!-- comment blah blah
still a comment , blah and a space at the end 
-->
<!rheum>
<?rhaponicum>
<randomtag spam="eggs"/>
</body>
</html>
"""
        f = StringIO(data)
        return data, f

    def test_encoding(self):
        from pullparser import PullParser
        from StringIO import StringIO
        data = "<a>&#1092;</a>"

        f = StringIO(data)
        p = PullParser(f, encoding="KOI8-R")
        p.get_tag("a")
        self.assert_(p.get_text() == "\xc6")

        f = StringIO(data)
        p = PullParser(f, encoding="UTF-8")
        p.get_tag("a")
        self.assert_(p.get_text() == "\xd1\x84")

#        response = urllib.addinfourl(f, {"content-type": "text/html; charset=XXX"}, req.get_full_url())
    def test_get_token(self):
        from pullparser import PullParser, NoMoreTokensError
        data, f = self.data_and_file()
        p = PullParser(f)
        self.assert_(
            p.get_token() == ("decl",
'''DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN"
"http://www.w3.org/TR/html4/strict.dtd"''', None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("starttag", "html", []))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("starttag", "head", []))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("starttag", "title", [("an", "attr")]))
        self.assert_(p.get_token() == ("data", "Title", None))
        self.assert_(p.get_token() == ("endtag", "title", None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("endtag", "head", None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("starttag", "body", []))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("starttag", "p", []))
        self.assert_(p.get_token() == ("data", "This is a data ", None))
        self.assert_(p.get_token() == ("starttag", "img", [("alt", "blah")]))
        self.assert_(p.get_token() == ("data", " ", None))
        self.assert_(p.get_token() == ("entityref", "amp", None))
        self.assert_(p.get_token() == ("data",
                                       " that was an entityref and this ",
                                       None))
        self.assert_(p.get_token() == ("charref", "097", None))
        self.assert_(p.get_token() == ("data", " is\na charref.  ", None))
        self.assert_(p.get_token() == ("starttag", "blah",
                                       [("foo", "bing"), ("blam", "wallop")]))
        self.assert_(p.get_token() == ("data", ".\n", None))
        self.assert_(p.get_token() == (
            "comment", " comment blah blah\n"
            "still a comment , blah and a space at the end \n", None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("decl", "rheum", None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("pi", "rhaponicum", None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("startendtag", "randomtag",
                                       [("spam", "eggs")]))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("endtag", "body", None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assert_(p.get_token() == ("endtag", "html", None))
        self.assert_(p.get_token() == ("data", "\n", None))
        self.assertRaises(NoMoreTokensError, p.get_token)
#        print "token", p.get_token()
#        sys.exit()

    def test_unget_token(self):
        from pullparser import PullParser, NoMoreTokensError
        data, f = self.data_and_file()
        p = PullParser(f)
        p.get_token()
        tok = p.get_token()
        self.assert_(tok == ("data", "\n", None))
        p.unget_token(tok)
        self.assert_(p.get_token() == ("data", "\n", None))
        tok = p.get_token()
        self.assert_(tok == ("starttag", "html", []))
        p.unget_token(tok)
        self.assert_(tok == ("starttag", "html", []))

    def test_get_tag(self):
        from pullparser import PullParser, NoMoreTokensError
        data, f = self.data_and_file()
        p = PullParser(f)
        self.assert_(p.get_tag() == ("starttag", "html", []))
        self.assert_(p.get_tag("blah", "body", "title") ==
                     ("starttag", "title", [("an", "attr")]))
        self.assert_(p.get_tag() == ("endtag", "title", None))
        self.assert_(p.get_tag("randomtag") == ("startendtag", "randomtag",
                                                [("spam", "eggs")]))
        self.assert_(p.get_tag() == ("endtag", "body", None))
        self.assert_(p.get_tag() == ("endtag", "html", None))
        self.assertRaises(NoMoreTokensError, p.get_tag)
#        print "tag", p.get_tag()
#        sys.exit()

    def test_get_text(self):
        from pullparser import PullParser, NoMoreTokensError
        data, f = self.data_and_file()
        p = PullParser(f)
        self.assert_(p.get_text() == "\n")
        self.assert_(peek_token(p).data == "html")
        self.assert_(p.get_text() == "")
        self.assert_(peek_token(p).data == "html"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        self.assert_(p.get_text() == "Title"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        self.assert_(p.get_text() == "This is a data blah[IMG]"); p.get_token()
        self.assert_(p.get_text() == " & that was an entityref "
                     "and this a is\na charref.  "); p.get_token()
        self.assert_(p.get_text() == ".\n\n\n\n"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        self.assert_(p.get_text() == "\n"); p.get_token()
        # no more tokens, so we just get empty string
        self.assert_(p.get_text() == "")
        self.assert_(p.get_text() == "")
        self.assertRaises(NoMoreTokensError, p.get_token)
        #print "text", `p.get_text()`
        #sys.exit()

    def test_get_text_2(self):
        # more complicated stuff

        from pullparser import PullParser, NoMoreTokensError

        # endat
        data, f = self.data_and_file()
        p = PullParser(f)
        self.assert_(p.get_text(endat=("endtag", "html")) ==
                     u"\n\n\nTitle\n\n\nThis is a data blah[IMG]"
                     " & that was an entityref and this a is\na charref.  ."
                     "\n\n\n\n\n\n")
        f.close()

        data, f = self.data_and_file()
        p = PullParser(f)
        self.assert_(p.get_text(endat=("endtag", "title")) ==
                     "\n\n\nTitle")
        self.assert_(p.get_text(endat=("starttag", "img")) ==
                     "\n\n\nThis is a data blah[IMG]")
        f.close()

        # textify arg
        data, f = self.data_and_file()
        p = PullParser(f, textify={"title": "an", "img": lambda x: "YYY"})
        self.assert_(p.get_text(endat=("endtag", "title")) ==
                     "\n\n\nattr[TITLE]Title")
        self.assert_(p.get_text(endat=("starttag", "img")) ==
                     "\n\n\nThis is a data YYY")
        f.close()

        # get_compressed_text
        data, f = self.data_and_file()
        p = PullParser(f)
        self.assert_(p.get_compressed_text(endat=("endtag", "html")) ==
                     u"Title This is a data blah[IMG]"
                     " & that was an entityref and this a is a charref. .")
        f.close()


if __name__ == "__main__":
    import unittest
    unittest.main()
